(function() {
    "use strict";

    angular.module("weChatApp")
        .factory("TranslationService", ["$q", "VeevaMessages", "Endpoints", TranslationService])
    ;

    function TranslationService($q, veevaMessages, endpoints) {
        var service = this;
        var translatedMessages;
        var translatedObjectLabels;
        var translatedFieldLabels;
        var translatedPicklistLabels;
        var translatedRecordTypeLabels;

        service.getFieldLabels = function(fieldLabelMap) {
            var deferred = $q.defer();
            var translationQueue = [];
            var translationPromises = [];

            for(var key in fieldLabelMap) {
                if(Object.prototype.hasOwnProperty.call(fieldLabelMap, key)) {
                    var subDeferred = $q.defer();

                    translationQueue.push({
                        object: key,
                        fields: fieldLabelMap[key],
                        deferred: subDeferred
                    });

                    translationPromises.push(subDeferred.promise);
                }
            }

            translationQueue.forEach(function(transBlock) {
                endpoints.fetchFieldLabels(transBlock.object, transBlock.fields).then(function(resp) {
                    transBlock.deferred.resolve(resp);
                }, rejectionHandler(transBlock.deferred));
            });

            $q.all(translationPromises).then(function(resp) {
                var translationResult = {};
                translationQueue.forEach(function(transBlock, i) {
                    transBlock.fields.forEach(function(transField, j) {
                        var objectName = transBlock.object;
                        var fieldName = transField;
                        if(!translationResult[objectName]) {
                            translationResult[objectName] = {};
                        }
                        translationResult[objectName][fieldName] = resp[i][j].display;
                    });
                });
                translatedFieldLabels = translationResult;
                deferred.resolve(translatedFieldLabels);
            }, rejectionHandler(deferred));

            return deferred.promise;
        };

        service.getObjectLabels = function(fieldLabelMap) {
            var deferred = $q.defer();
            var objectList = [];

            for(var key in fieldLabelMap) {
                if(Object.prototype.hasOwnProperty.call(fieldLabelMap, key)) {
                    objectList.push(key);
                }
            }

            endpoints.fetchObjectLabels(objectList).then(function(resp) {
                translatedObjectLabels = setupTranslatedObjectLabels(resp);
                deferred.resolve(translatedObjectLabels);
            }, rejectionHandler(deferred));

            return deferred.promise;
        };

        service.getPicklistLabels = function(picklistLabelMap) {
            var deferred = $q.defer();
            var translationQueue = [];
            var translationPromises = [];

            for(var key in picklistLabelMap) {
                if(Object.prototype.hasOwnProperty.call(picklistLabelMap, key)) {
                    var picklistFields = picklistLabelMap[key];
                    picklistFields.forEach(function(pField) {
                        var subDeferred = $q.defer();
                        translationQueue.push({
                            object: key,
                            field: pField,
                            deferred: subDeferred
                        });
                        translationPromises.push(subDeferred.promise);
                    });
                }
            }

            translationQueue.forEach(function(transBlock) {
                endpoints.fetchPicklistLabels(transBlock.object, transBlock.field).then(function(resp) {
                    transBlock.deferred.resolve(resp);
                }, rejectionHandler(transBlock.deferred));
            });

            $q.all(translationPromises).then(function(combinedResp) {
                var translationResult = {};
                translationQueue.forEach(function(transBlock, i) {
                    var objectName = transBlock.object;
                    var fieldName = transBlock.field;
                    if(!translationResult[objectName]) {
                        translationResult[objectName] = {};
                    }
                    translationResult[objectName][fieldName] = combinedResp[i][objectName][fieldName];
                });
                translatedPicklistLabels = translationResult;
                deferred.resolve(translatedPicklistLabels);
            }, rejectionHandler(deferred));

            return deferred.promise;
        };

        service.getRecordTypeLabels = function(recordTypeLabelList) {
            var deferred = $q.defer();
            var translationQueue = [];
            var translationPromises = [];

            recordTypeLabelList.forEach(function(objectName) {
                var subDeferred = $q.defer();
                translationQueue.push({
                    object: objectName,
                    deferred: subDeferred
                });
                translationPromises.push(subDeferred.promise);
            });

            translationQueue.forEach(function(transBlock) {
                endpoints.fetchRecordTypeLabels(transBlock.object).then(function(resp) {
                    transBlock.deferred.resolve(resp);
                }, rejectionHandler(transBlock.deferred));
            });

            $q.all(translationPromises).then(function(combinedResp) {
                var translationResult = {};
                translationQueue.forEach(function(transBlock, i) {
                    var objectName = transBlock.object;
                    translationResult[objectName] = combinedResp[i][objectName];
                });
                translatedRecordTypeLabels = translationResult;
                deferred.resolve(translatedRecordTypeLabels);
            }, rejectionHandler(deferred));

            return deferred.promise;
        };

        service.getVeevaMessages = function(locale, defaultLocale) {
            var deferred = $q.defer();
            var messageTokens = _.values(veevaMessages);

            endpoints.fetchVeevaMessages(messageTokens, locale).then(function(resp) {
                if(_.values(resp).length === 0) {
                    endpoints.fetchVeevaMessages(messageTokens, defaultLocale).then(function(resp) {
                        translatedMessages = setupTranslatedMessages(resp);
                        deferred.resolve(translatedMessages);
                    });
                } else {
                    translatedMessages = setupTranslatedMessages(resp);
                    deferred.resolve(translatedMessages);
                }
            }, rejectionHandler(deferred));

            return deferred.promise;
        };

        /**
         * Method used by message filter to show translated message on the UI
         * Accepted parameters include:
         * 1. message key as string
         *      e.g. 'all'
         * 2. an Array contains the message key as the first element, follow by any replaceable values
         *      e.g. ['progress', '50', '100']
         *      note: the message for 'progress' could be: "{0} out of {1} completed"
         * @returns {*}
         */
        service.getMessage = function() {
            var key = arguments[0];
            if(translatedMessages[key]) {
                var message = translatedMessages[key];
                for(var i = 1; i < arguments.length; i++) {
                    var token = arguments[i];
                    var marker = i - 1;
                    message = message.replace('{' + marker + '}', token);
                }
                return message;
            } else {
                return key;
            }
        };

        function setupTranslatedMessages(veevaMessageResp) {
            var translatedMsg = {};

            angular.forEach(veevaMessages, function(msgToken, key) {
                var foundTranslation = _.find(veevaMessageResp, function(rawMsg) {
                    return msgToken.msgName === rawMsg.Name.value && msgToken.msgCategory === rawMsg.Category_vod__c.value;
                });

                if(foundTranslation) {
                    translatedMsg[key] = foundTranslation.Text_vod__c.display;
                }
            });

            return translatedMsg;
        }

        function setupTranslatedObjectLabels(objLabelResp) {
            var translatedObjLabels = {};

            angular.forEach(objLabelResp, function(msgToken, key) {
                if(typeof msgToken === 'object' && msgToken.length > 0) {
                    translatedObjLabels[key] = msgToken[0];
                }
            });

            return translatedObjLabels;
        }

        function rejectionHandler(deferred) {
            return function(error) {
                console.error(error);
                deferred.reject(error);
            };
        }

        return service;
    }
}());